﻿namespace histcopy
{
    internal class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("histcopy version 2.01 (2024-05-12) (c) 2002-2024 Kazuhiro Takahashi");
            
            // 引数解析
            var directories = new List<string>();
            var header = 4096;
            var footer = 4096;
            var buffer = 1024 * 1024;
            var rate = 0.01;
            var optionSwitch = "";
            try
            {
                foreach (var arg in args)
                {
                    if (arg.StartsWith('-'))
                        optionSwitch = arg[1..];
                    else
                    {
                        switch (optionSwitch)
                        {
                            case "": directories.Add(arg); break;
                            case "header": header = ToInt(arg, 0, 1024 * 1024); break;
                            case "footer": footer = ToInt(arg, 0, 1024 * 1024); break;
                            case "buffer": buffer = ToInt(arg, 4096, 256 * 1024 * 1024); break;
                            case "rate": rate = ToDouble(arg, 0.0, 1.0); break;
                            default: throw new Exception();
                        }
                        optionSwitch = "";
                    }
                }
                if (directories.Count != 2)
                    throw new Exception();
            }
            catch
            {
                // エラーのときはヘルプ表示
                Console.WriteLine("usage: histcopy <source> <destination> [options...]");
                Console.WriteLine("default: -header 4K -footer 4K -buffer 1M -rate 1%");
                return;
            }

            try {
                var source = new CancellationTokenSource();
                var copy = new Histcopy(directories[0], directories[1], source.Token) {
                    HeaderLength = header,
                    FooterLength = footer,
                    BufferLength = buffer,
                    VerifyRate = rate
                };
                copy.Progress += (s, e) => Console.WriteLine(e.Message);
                Console.CancelKeyPress += (s, e) => {
                    e.Cancel = true;
                    source.Cancel();
                    Console.Write("^C");
                };
                copy.Run();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        private static int ToInt(string text, int min, int max)
        {
            int result;
            if (text.EndsWith('M'))
                result = int.Parse(text[..^1]) * 1024 * 1024;
            else if (text.EndsWith('K'))
                result = int.Parse(text[..^1]) * 1024;
            else
                result = int.Parse(text);
            if (result < min || result > max)
                throw new Exception();
            return result;
        }

        private static double ToDouble(string text, double min, double max)
        {
            double result;
            if (text.EndsWith('%'))
                result = double.Parse(text[..^1]) / 100.0;
            else
                result = double.Parse(text);
            if (result < min || result > max)
                throw new Exception();
            return result;
        }
    }
}
